from os import PathLike
from typing import Union
from functools import wraps
import time
import logging

try:
    from ratelimit import RateLimitException
except ImportError:
    pass

_log = logging.getLogger('arya_api_framework.Sync')


def chunk_file_reader(file: Union[str, PathLike[str]]):
    with open(file, 'rb') as f:
        chunk = f.read(64 * 1024)

        while chunk:
            yield chunk
            chunk = f.read(64 * 1024)


def sleep_and_retry(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        while True:
            try:
                return func(*args, **kwargs)
            except RateLimitException as exception:
                _log.info(f"Applying rate limit: Sleeping for {exception.period_remaining}s")
                time.sleep(exception.period_remaining)
    return wrapper



from .async_framework import AsyncClient
from .sync_framework import SyncClient
from .framework import Response, PaginatedResponse

__all__ = {
    "AsyncClient",
    "SyncClient",
    "Response",
    "PaginatedResponse"
}



import abc
from typing import Any, Optional
from datetime import datetime

from pydantic import BaseModel, Field

from .errors import MISSING


class ClientInit(type):
    def __call__(cls, *args, **kwargs) -> Any:
        uri = kwargs.get('uri', MISSING)
        headers = kwargs.get('headers', MISSING)
        cookies = kwargs.get('cookies', MISSING)
        parameters = kwargs.get('parameters', MISSING)
        error_responses = kwargs.get('error_responses', MISSING)
        bearer_token = kwargs.get('bearer_token', MISSING)
        rate_limit = kwargs.get('rate_limit', MISSING)
        rate_limit_interval = kwargs.get('rate_limit_interval', MISSING)

        obj = type.__call__(
            cls, uri=uri, headers=headers, cookies=cookies, parameters=parameters,
            error_responses=error_responses, bearer_token=bearer_token, rate_limit=rate_limit,
            rate_limit_interval=rate_limit_interval
        )
        if hasattr(obj, '__post_init__'):
            obj.__post_init__(*args, **kwargs)
        return obj


class Response(BaseModel, abc.ABC):
    request_base_: Optional[str] = None
    request_received_at_: Optional[datetime] = Field(default_factory=datetime.utcnow)


class PaginatedResponse(Response, abc.ABC):
    @property
    @abc.abstractmethod
    def is_paginating(self) -> bool:
        return False

    @property
    @abc.abstractmethod
    def next(self) -> Optional[int]:
        return None

    @property
    @abc.abstractmethod
    def end(self) -> Optional[int]:
        return None

    @property
    @abc.abstractmethod
    def back(self) -> Optional[int]:
        return None

    @property
    @abc.abstractmethod
    def start(self) -> Optional[int]:
        return None



from typing import Any, Type, Union

from pydantic import validate_arguments

from .errors import ValidationError


@validate_arguments()
def validate_type(param: Any, target: Union[Type, list[Type]], err: bool = True) -> bool:
    if isinstance(target, list):
        for t in target:
            if isinstance(param, t):
                return True
            if type(param) is t:
                return True
            if issubclass(type(param), t):
                return True
    else:
        if isinstance(param, target):
            return True
        if type(param) is target:
            return True
        if issubclass(type(param), target):
            return True

    if err:
        raise ValidationError(f"{param} is not of type {target}.")

    return False



import logging
from datetime import timedelta, datetime
from typing import Any, Optional, Type, TypeVar, Union, Dict, List
from json import JSONDecodeError

from multidict import CIMultiDict
from pydantic import BaseModel, parse_obj_as, SecretStr, validate_arguments
from yarl import URL

from ..errors import HTTPError, ResponseParseError, error_response_mapping, MISSING, AsyncClientError
from ..framework import ClientInit, Response
from ..utils import validate_type
from .utils import chunk_file_reader, merge_params

is_async: bool
try:
    import asyncio
    from aiohttp import ClientSession, ClientTimeout
    from aiolimiter import AsyncLimiter

    is_async = True
except ImportError:
    is_async = False

__all__ = {
    "AsyncClient"
}

_log: logging.Logger = logging.getLogger("arya_api_framework.Async")

MappingOrModel = Union[Dict[str, Union[str, int]], BaseModel]
HttpMapping = Dict[str, Union[str, int, List[Union[str, int]]]]
Parameters = Union[HttpMapping, BaseModel]
Cookies = MappingOrModel
Headers = MappingOrModel
Body = Union[Dict[str, Any], BaseModel]
ErrorResponses = Dict[int, Type[BaseModel]]

ClientSessionT = TypeVar('ClientSessionT', bound='ClientSession')


class AsyncClient(metaclass=ClientInit):
    """The basic Client implementation that all API clients inherit from."""

    _headers: Optional[Headers] = None
    _cookies: Optional[Cookies] = None
    _parameters: Optional[Parameters] = None
    _error_responses: Optional[ErrorResponses] = None
    _rate_limit_interval: Optional[Union[int, float]] = 1
    _rate_limit: Optional[Union[int, float]] = None
    _limiter: Optional['AsyncLimiter'] = None
    _last_request_at: Optional[datetime] = None
    _base: Optional[URL] = MISSING
    _session: ClientSessionT

    # ---------- Initialization Methods ----------
    def __init__(
            self,
            /,
            uri: str = MISSING,
            headers: Headers = MISSING,
            cookies: Cookies = MISSING,
            parameters: Parameters = MISSING,
            error_responses: ErrorResponses = MISSING,
            bearer_token: Union[str, SecretStr] = MISSING,
            rate_limit: Union[int, float] = MISSING,
            rate_limit_interval: Union[int, float] = MISSING
    ) -> None:
        if not is_async:
            raise AsyncClientError("The async context is unavailable. Try installing with `python -m pip install arya-api-framework[async]`.")

        if uri is not MISSING:
            if not isinstance(uri, str):
                raise ValueError("The uri should be a string.")
            self._base = URL(uri)

        if self.uri is None:
            raise AsyncClientError(
                "The client needs a base uri specified. "
                "This can be done through init parameters, or subclass parameters."
            )

        if bearer_token is not None:
            if isinstance(bearer_token, SecretStr):
                bearer_token = bearer_token.get_secret_value()

            if headers is None or headers is MISSING:
                headers = {}

            headers["Authorization"] = f"Bearer {bearer_token}"

        if headers is not MISSING:
            self._headers = headers or {}
        if cookies is not MISSING:
            self._cookies = cookies or {}
        if parameters is not MISSING:
            self._parameters = parameters or {}

        if error_responses is not MISSING:
            self._error_responses = error_responses

        if rate_limit is not MISSING:
            if validate_type(rate_limit, [int, float]):
                self._rate_limit = rate_limit
        if rate_limit_interval is not MISSING:
            if validate_type(rate_limit_interval, [int, float]):
                self._rate_limit_interval = rate_limit_interval

        if self._rate_limit:
            self._limiter = AsyncLimiter(self._rate_limit, self._rate_limit_interval)

        self._session = ClientSession(
            self.uri_root,
            headers=self.headers or {},
            cookies=self.cookies or {}
        )

    def __post_init__(self, *args, **kwargs) -> None:
        pass

    def __init_subclass__(
            cls,
            uri: str = MISSING,
            headers: Headers = MISSING,
            cookies: Cookies = MISSING,
            parameters: Parameters = MISSING,
            error_responses: ErrorResponses = MISSING,
            rate_limit: Union[int, float] = MISSING,
            rate_limit_interval: Union[int, float] = MISSING
    ) -> None:
        if uri is not MISSING:
            if not isinstance(uri, str):
                raise ValueError("The uri should be a string.")
            cls._base = URL(uri)
        if headers is not MISSING:
            cls._headers = headers
        if cookies is not MISSING:
            cls._cookies = cookies or {}
        if parameters is not MISSING:
            cls._parameters = parameters or {}
        if error_responses is not MISSING:
            cls._error_responses = error_responses
        if rate_limit is not MISSING:
            if validate_type(rate_limit, [int, float]):
                cls._rate_limit = rate_limit
        if rate_limit_interval is not MISSING:
            if validate_type(rate_limit_interval, [int, float]):
                cls._rate_limit_interval = rate_limit_interval

    # ---------- URI Options ----------
    @property
    def uri(self) -> Optional[str]:
        return str(self._base) if self._base is not MISSING else None

    @property
    def uri_root(self) -> Optional[str]:
        return str(self._base.origin()) if self._base is not MISSING else None

    @property
    def uri_rel(self) -> Optional[str]:
        return str(self._base.relative()) if self._base is not MISSING else None

    # ---------- Default Request Settings ----------
    @property
    def headers(self) -> Optional[Headers]:
        return self._headers

    @headers.setter
    def headers(self, headers: Headers) -> None:
        self._headers = self._flatten_format(headers)
        self._session._default_headers = CIMultiDict(headers) if headers else CIMultiDict()

    @property
    def cookies(self) -> Optional[Cookies]:
        return self._cookies

    @cookies.setter
    def cookies(self, cookies: Cookies) -> None:
        self._cookies = self._flatten_format(cookies)
        self._session._cookie_jar.update_cookies(self._cookies)

    @property
    def parameters(self) -> Optional[Parameters]:
        return self._parameters

    @parameters.setter
    def parameters(self, params: Parameters) -> None:
        self._parameters = self._flatten_format(params)

    @property
    def rate_limit(self) -> Optional[timedelta]:
        if self._rate_limit:
            return timedelta(seconds=self._rate_limit_interval/self._rate_limit)

    @property
    def error_responses(self) -> Optional[ErrorResponses]:
        return self._error_responses

    @error_responses.setter
    @validate_arguments()
    def error_responses(self, error_responses: ErrorResponses) -> None:
        if error_responses is not MISSING:
            self._error_responses = error_responses

    # ---------- Request Methods ----------
    @validate_arguments()
    async def request(
            self,
            method: str,
            path: str = None,
            *,
            body: Body = None,
            data: Any = None,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        if self._limiter:
            if not self._limiter.has_capacity():
                _log.info("Waiting for rate limit")
            await self._limiter.acquire()

        path = self.uri_rel + path if self.uri_rel else path
        headers = self._flatten_format(headers)
        cookies = self._flatten_format(cookies)
        parameters = merge_params(self.parameters, self._flatten_format(parameters))
        body = self._flatten_format(body)
        error_responses = error_responses or self.error_responses or {}

        async with self._session.request(
                method,
                path,
                headers=headers,
                cookies=cookies,
                params=parameters,
                json=body,
                data=data,
                timeout=ClientTimeout(total=timeout)
        ) as response:
            _log.info(f"[{method} {response.status}] {path} {URL(response.url).query_string}")

            if response.ok:
                try:
                    response_json = await response.json(content_type=None)
                except JSONDecodeError:
                    response_text = await response.text()
                    raise ResponseParseError(raw_response=response_text)

                if response_format is not None:
                    obj = parse_obj_as(response_format, response_json)
                    obj.request_base_ = str(response.url)
                    return obj

                return response_json

            error_class = error_response_mapping.get(response.status, HTTPError)
            error_response_model = error_responses.get(response.status)

            try:
                response_json = await response.json(content_type=None)
            except JSONDecodeError:
                response_text = await response.text()
                raise ResponseParseError(raw_response=response_text)

            if bool(error_response_model):
                raise error_class(parse_obj_as(error_response_model, response_json))

            raise error_class(response_json)

    @validate_arguments()
    async def upload_file(
            self,
            path: str,
            file: str,
            *,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.post(
            path,
            headers=headers,
            cookies=cookies,
            parameters=parameters,
            data={'file': open(file, 'rb')},
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses,
        )

    @validate_arguments()
    async def stream_file(
            self,
            path: str,
            file: str,
            *,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,  # Default in aiohttp
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.post(
            path,
            headers=headers,
            cookies=cookies,
            parameters=parameters,
            data=chunk_file_reader(file),
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses
        )

    @validate_arguments()
    async def get(
            self,
            path: str = None,
            *,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,  # Default in aiohttp
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.request(
            "GET",
            path,
            headers=headers,
            cookies=cookies,
            parameters=parameters,
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses,
        )

    @validate_arguments()
    async def post(
            self,
            path: str = None,
            *,
            body: Body = None,
            data: Any = None,
            headers: Headers = None,
            cookies: Cookies = None,
            params: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,  # Default in aiohttp
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.request(
            "POST",
            path,
            body=body,
            data=data,
            headers=headers,
            cookies=cookies,
            params=params,
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses,
        )

    @validate_arguments()
    async def patch(
            self,
            path: str = None,
            *,
            body: Body = None,
            data: Any = None,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,  # Default in aiohttp
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.request(
            "PATCH",
            path,
            body=body,
            data=data,
            headers=headers,
            cookies=cookies,
            parameters=parameters,
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses,
        )

    @validate_arguments()
    async def put(
            self,
            path: str = None,
            *,
            body: Body = None,
            data: Any = None,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,  # Default in aiohttp
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.request(
            "PUT",
            path,
            body=body,
            data=data,
            headers=headers,
            cookies=cookies,
            parameters=parameters,
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses,
        )

    @validate_arguments()
    async def delete(
            self,
            path: str = None,
            *,
            body: Body = None,
            data: Any = None,
            headers: Headers = None,
            cookies: Cookies = None,
            parameters: Parameters = None,
            response_format: Type[Response] = None,
            timeout: int = 300,  # Default in aiohttp
            error_responses: ErrorResponses = None
    ) -> Optional[Response]:
        return await self.request(
            "DELETE",
            path,
            body=body,
            data=data,
            headers=headers,
            cookies=cookies,
            parameters=parameters,
            response_format=response_format,
            timeout=timeout,
            error_responses=error_responses,
        )

    async def close(self):
        await self._session.close()

    # ---------- Class Methods ----------
    @classmethod
    @validate_arguments()
    def _flatten_format(cls, data: Optional[Parameters]) -> Dict[str, Any]:
        return data.dict(exclude_unset=True) if isinstance(data, BaseModel) else data



from os import PathLike
from typing import Union, Optional, Dict, Mapping, List
from collections import OrderedDict

from ..errors import AsyncClientError

is_async: bool
try:
    import aiofiles
    is_async = True
except ImportError:
    is_async = False


async def chunk_file_reader(file: Union[str, PathLike[str]]):
    if not is_async:
        raise AsyncClientError("The async context is unavailable. Try installing with `python -m pip install arya-api-framework[async]`.")

    async with aiofiles.open(file, 'rb') as f:
        chunk = await f.read(64 * 1024)

        while chunk:
            yield chunk
            chunk = await f.read(64 * 1024)


def _to_key_val_list(obj: Optional[Dict]) -> Optional[List]:
    if obj is None:
        return obj

    if isinstance(obj, (str, bytes, bool, int)):
        raise ValueError("Cannot encode objects that are not key-val paired.")

    if isinstance(obj, Mapping):
        obj = obj.items()

    return list(obj)


def merge_params(static_params: Optional[Dict], request_params: Optional[Dict]) -> Optional[Mapping]:
    if request_params is None:
        return static_params

    if static_params is None:
        return request_params

    if not (isinstance(static_params, Mapping) and isinstance(request_params, Mapping)):
        return request_params

    merged = OrderedDict(_to_key_val_list(static_params))
    merged.update(_to_key_val_list(request_params))

    none_keys = [k for (k, v) in merged.items() if v is None]
    for k in none_keys:
        del merged[k]

    return merged



from typing import Union

from pydantic import BaseModel

from .statuses import *

Response = Union[str, bytes, dict, list, BaseModel]
RawResponse = Union[str, bytes]


# ---------- Framework ----------
class MISSING(Exception):
    """Used to show when something is missing.

    This is a preferred usage in this program over a NoneType alternative,
    which is used often as an actual data type.
    """
    def __bool__(self) -> bool:
        return False


class ValidationError(Exception):
    """Used when validating a variables type."""


# ---------- Client Errors ----------
class ClientError(Exception):
    pass


class AsyncClientError(ClientError):
    pass


class SyncClientError(ClientError):
    pass


class ResponseParseError(ClientError):
    def __init__(self, raw_response: RawResponse):
        self.raw_response = raw_response


# ---------- Request Errors ----------
class HTTPError(Exception):
    status_code: int = None
    response: Response = None

    def __init__(self, response: Response):
        self.response = response


class HTTPRedirect(HTTPError):
    pass


class HTTPClientError(HTTPError):
    pass


class HTTPServerError(HTTPError):
    pass


# 300
class HTTPMultipleChoices(HTTPRedirect):
    status_code = HTTP_300_MULTIPLE_CHOICES


class HTTPMovedPermanently(HTTPRedirect):
    status_code = HTTP_301_MOVED_PERMANENTLY


class HTTPFound(HTTPRedirect):
    status_code = HTTP_302_FOUND


class HTTPSeeOther(HTTPRedirect):
    status_code = HTTP_303_SEE_OTHER


class HTTPNotModified(HTTPRedirect):
    status_code = HTTP_304_NOT_MODIFIED


class HTTPUseProxy(HTTPRedirect):
    status_code = HTTP_305_USE_PROXY


class HTTPReserved(HTTPRedirect):
    status_code = HTTP_306_RESERVED


class HTTPTemporaryRedirect(HTTPRedirect):
    status_code = HTTP_307_TEMPORARY_REDIRECT


class HTTPPermanentRedirect(HTTPRedirect):
    status_code = HTTP_308_PERMANENT_REDIRECT


# 400
class HTTPBadRequest(HTTPClientError):
    status_code = HTTP_400_BAD_REQUEST


class HTTPUnauthorized(HTTPClientError):
    status_code = HTTP_401_UNAUTHORIZED


class HTTPPaymentRequired(HTTPClientError):
    status_code = HTTP_402_PAYMENT_REQUIRED


class HTTPForbidden(HTTPClientError):
    status_code = HTTP_403_FORBIDDEN


class HTTPNotFound(HTTPClientError):
    status_code = HTTP_404_NOT_FOUND


class HTTPMethodNotAllowed(HTTPClientError):
    status_code = HTTP_405_METHOD_NOT_ALLOWED


class HTTPNotAcceptable(HTTPClientError):
    status_code = HTTP_406_NOT_ACCEPTABLE


class HTTPProxyAuthenticationRequired(HTTPClientError):
    status_code = HTTP_407_PROXY_AUTHENTICATION_REQUIRED


class HTTPRequestTimeout(HTTPClientError):
    status_code = HTTP_408_REQUEST_TIMEOUT


class HTTPConflict(HTTPClientError):
    status_code = HTTP_409_CONFLICT


class HTTPGone(HTTPClientError):
    status_code = HTTP_410_GONE


class HTTPLengthRequired(HTTPClientError):
    status_code = HTTP_411_LENGTH_REQUIRED


class HTTPPreconditionFailed(HTTPClientError):
    status_code = HTTP_412_PRECONDITION_FAILED


class HTTPRequestEntityTooLarge(HTTPClientError):
    status_code = HTTP_413_REQUEST_ENTITY_TOO_LARGE


class HTTPRequestUriTooLong(HTTPClientError):
    status_code = HTTP_414_REQUEST_URI_TOO_LONG


class HTTPUnsupportedMediaType(HTTPClientError):
    status_code = HTTP_415_UNSUPPORTED_MEDIA_TYPE


class HTTPRequestedRangeNotSatisfiable(HTTPClientError):
    status_code = HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE


class HTTPExpectationFailed(HTTPClientError):
    status_code = HTTP_417_EXPECTATION_FAILED


class HTTPImATeapot(HTTPClientError):
    status_code = HTTP_418_IM_A_TEAPOT


class HTTPMisdirectedRequest(HTTPClientError):
    status_code = HTTP_421_MISDIRECTED_REQUEST


class HTTPUnprocessableEntity(HTTPClientError):
    status_code = HTTP_422_UNPROCESSABLE_ENTITY


class HTTPLocked(HTTPClientError):
    status_code = HTTP_423_LOCKED


class HTTPFailedDependency(HTTPClientError):
    status_code = HTTP_424_FAILED_DEPENDENCY


class HTTPTooEarly(HTTPClientError):
    status_code = HTTP_425_TOO_EARLY


class HTTPUpgradeRequired(HTTPClientError):
    status_code = HTTP_426_UPGRADE_REQUIRED


class HTTPPreconditionRequired(HTTPClientError):
    status_code = HTTP_428_PRECONDITION_REQUIRED


class HTTPTooManyRequests(HTTPClientError):
    status_code = HTTP_429_TOO_MANY_REQUESTS


class HTTPRequestHeaderFieldsTooLarge(HTTPClientError):
    status_code = HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE


class HTTPUnavailableForLegalReasons(HTTPClientError):
    status_code = HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS


# 500
class HTTPInternalServerError(HTTPServerError):
    status_code = HTTP_500_INTERNAL_SERVER_ERROR


class HTTPNotImplemented(HTTPServerError):
    status_code = HTTP_501_NOT_IMPLEMENTED


class HTTPBadGateway(HTTPServerError):
    status_code = HTTP_502_BAD_GATEWAY


class HTTPServiceUnavailable(HTTPServerError):
    status_code = HTTP_503_SERVICE_UNAVAILABLE


class HTTPGatewayTimeout(HTTPServerError):
    status_code = HTTP_504_GATEWAY_TIMEOUT


class HTTPHttpServerVersionNotSupported(HTTPServerError):
    status_code = HTTP_505_HTTP_VERSION_NOT_SUPPORTED


class HTTPVariantAlsoNegotiates(HTTPServerError):
    status_code = HTTP_506_VARIANT_ALSO_NEGOTIATES


class HTTPInsufficientStorage(HTTPServerError):
    status_code = HTTP_507_INSUFFICIENT_STORAGE


class HTTPLoopDetected(HTTPServerError):
    status_code = HTTP_508_LOOP_DETECTED


class HTTPNotExtended(HTTPServerError):
    status_code = HTTP_510_NOT_EXTENDED


class HTTPNetworkAuthenticationRequired(HTTPServerError):
    status_code = HTTP_511_NETWORK_AUTHENTICATION_REQUIRED


# Map errors codes to their related exception classes.
error_response_mapping = {
    # 300
    HTTP_300_MULTIPLE_CHOICES: HTTPMultipleChoices,
    HTTP_301_MOVED_PERMANENTLY: HTTPMovedPermanently,
    HTTP_302_FOUND: HTTPFound,
    HTTP_303_SEE_OTHER: HTTPSeeOther,
    HTTP_304_NOT_MODIFIED: HTTPNotModified,
    HTTP_305_USE_PROXY: HTTPUseProxy,
    HTTP_306_RESERVED: HTTPReserved,
    HTTP_307_TEMPORARY_REDIRECT: HTTPTemporaryRedirect,
    HTTP_308_PERMANENT_REDIRECT: HTTPPermanentRedirect,
    # 400
    HTTP_400_BAD_REQUEST: HTTPBadRequest,
    HTTP_401_UNAUTHORIZED: HTTPUnauthorized,
    HTTP_402_PAYMENT_REQUIRED: HTTPPaymentRequired,
    HTTP_403_FORBIDDEN: HTTPForbidden,
    HTTP_404_NOT_FOUND: HTTPNotFound,
    HTTP_405_METHOD_NOT_ALLOWED: HTTPMethodNotAllowed,
    HTTP_406_NOT_ACCEPTABLE: HTTPNotAcceptable,
    HTTP_407_PROXY_AUTHENTICATION_REQUIRED: HTTPProxyAuthenticationRequired,
    HTTP_408_REQUEST_TIMEOUT: HTTPRequestTimeout,
    HTTP_409_CONFLICT: HTTPConflict,
    HTTP_410_GONE: HTTPGone,
    HTTP_411_LENGTH_REQUIRED: HTTPLengthRequired,
    HTTP_412_PRECONDITION_FAILED: HTTPPreconditionFailed,
    HTTP_413_REQUEST_ENTITY_TOO_LARGE: HTTPRequestEntityTooLarge,
    HTTP_414_REQUEST_URI_TOO_LONG: HTTPRequestUriTooLong,
    HTTP_415_UNSUPPORTED_MEDIA_TYPE: HTTPUnsupportedMediaType,
    HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE: HTTPRequestedRangeNotSatisfiable,
    HTTP_417_EXPECTATION_FAILED: HTTPExpectationFailed,
    HTTP_418_IM_A_TEAPOT: HTTPImATeapot,
    HTTP_421_MISDIRECTED_REQUEST: HTTPMisdirectedRequest,
    HTTP_422_UNPROCESSABLE_ENTITY: HTTPUnprocessableEntity,
    HTTP_423_LOCKED: HTTPLocked,
    HTTP_424_FAILED_DEPENDENCY: HTTPFailedDependency,
    HTTP_425_TOO_EARLY: HTTPTooEarly,
    HTTP_426_UPGRADE_REQUIRED: HTTPUpgradeRequired,
    HTTP_428_PRECONDITION_REQUIRED: HTTPPreconditionRequired,
    HTTP_429_TOO_MANY_REQUESTS: HTTPTooManyRequests,
    HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE: HTTPRequestHeaderFieldsTooLarge,
    HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS: HTTPUnavailableForLegalReasons,
    # 500
    HTTP_500_INTERNAL_SERVER_ERROR: HTTPInternalServerError,
    HTTP_501_NOT_IMPLEMENTED: HTTPNotImplemented,
    HTTP_502_BAD_GATEWAY: HTTPBadGateway,
    HTTP_503_SERVICE_UNAVAILABLE: HTTPServiceUnavailable,
    HTTP_504_GATEWAY_TIMEOUT: HTTPGatewayTimeout,
    HTTP_505_HTTP_VERSION_NOT_SUPPORTED: HTTPHttpServerVersionNotSupported,
    HTTP_506_VARIANT_ALSO_NEGOTIATES: HTTPVariantAlsoNegotiates,
    HTTP_507_INSUFFICIENT_STORAGE: HTTPInsufficientStorage,
    HTTP_508_LOOP_DETECTED: HTTPLoopDetected,
    HTTP_510_NOT_EXTENDED: HTTPNotExtended,
    HTTP_511_NETWORK_AUTHENTICATION_REQUIRED: HTTPNetworkAuthenticationRequired,
}




from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

requirements = [
    "pydantic>=1.9.1",
    "yarl>=1.7.2"
]

extras_require = {
    "sync": ["requests>=2.27.1", "ratelimit>=2.2.1"],
    "async": ["aiohttp>=3.8.1", "aiolimiter>=1.0.0"],
    "all": [
        "requests>=2.27.1",
        "aiohttp>=3.8.1",
    ]
}

packages = [
    "arya_api_framework",
    "arya_api_framework.async_framework",
    "arya_api_framework.sync_framework",
]

setup(
    name="arya-api-framework",
    version="0.1.4",
    description="A simple API framework used in many other API clients I create.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Aryathel/ApiFramework",
    author="Aryathel",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.10.0",
    packages=packages,
    install_requires=requirements,
    extras_require=extras_require,
    include_package_data=True
)
